@@ -4,7 +4,6 @@
 
 # python std lib
 import datetime
-import imp
 import json
 import logging
 import os
@@ -12,6 +11,8 @@ import re
 import sys
 import traceback
 import time
+from io import open
+from importlib.machinery import SourceFileLoader
 
 # pyKwalify imports
 import pykwalify
@@ -21,8 +22,9 @@ from pykwalify.rule import Rule
 from pykwalify.types import is_scalar, is_string, tt
 
 # 3rd party imports
-from pykwalify.compat import yaml
 from dateutil.parser import parse
+from pykwalify.compat import yml
+from ruamel.yaml.constructor import Constructor
 
 log = logging.getLogger(__name__)
 
@@ -31,7 +33,7 @@ class Core(object):
     """ Core class of pyKwalify """
 
     def __init__(self, source_file=None, schema_files=None, source_data=None, schema_data=None, extensions=None, strict_rule_validation=False,
-                 fix_ruby_style_regex=False, allow_assertions=False,):
+                 fix_ruby_style_regex=False, allow_assertions=False, file_encoding=None, schema_file_obj=None, data_file_obj=None):
         """
         :param extensions:
             List of paths to python files that should be imported and available via 'func' keywork.
@@ -61,15 +63,41 @@ class Core(object):
         self.fix_ruby_style_regex = fix_ruby_style_regex
         self.allow_assertions = allow_assertions
 
+        # Patch in all the normal python types into the yaml load instance so we can use all the
+        # internal python types in the yaml loading.
+        yml.constructor.add_constructor('tag:yaml.org,2002:python/bool', Constructor.construct_yaml_bool)
+        yml.constructor.add_constructor('tag:yaml.org,2002:python/complex', Constructor.construct_python_complex)
+        yml.constructor.add_constructor('tag:yaml.org,2002:python/dict', Constructor.construct_yaml_map)
+        yml.constructor.add_constructor('tag:yaml.org,2002:python/float', Constructor.construct_yaml_float)
+        yml.constructor.add_constructor('tag:yaml.org,2002:python/int', Constructor.construct_yaml_int)
+        yml.constructor.add_constructor('tag:yaml.org,2002:python/list', Constructor.construct_yaml_seq)
+        yml.constructor.add_constructor('tag:yaml.org,2002:python/long', Constructor.construct_python_long)
+        yml.constructor.add_constructor('tag:yaml.org,2002:python/none', Constructor.construct_yaml_null)
+        yml.constructor.add_constructor('tag:yaml.org,2002:python/str', Constructor.construct_python_str)
+        yml.constructor.add_constructor('tag:yaml.org,2002:python/tuple', Constructor.construct_python_tuple)
+        yml.constructor.add_constructor('tag:yaml.org,2002:python/unicode', Constructor.construct_python_unicode)
+
+        if data_file_obj:
+            try:
+                self.source = yml.load(data_file_obj.read())
+            except Exception as e:
+                raise CoreError("Unable to load data_file_obj input")
+
+        if schema_file_obj:
+            try:
+                self.schema = yml.load(schema_file_obj.read())
+            except Exception as e:
+                raise CoreError("Unable to load schema_file_obj")
+
         if source_file is not None:
             if not os.path.exists(source_file):
                 raise CoreError(u"Provided source_file do not exists on disk: {0}".format(source_file))
 
-            with open(source_file, "r") as stream:
+            with open(source_file, "r", encoding=file_encoding) as stream:
                 if source_file.endswith(".json"):
                     self.source = json.load(stream)
                 elif source_file.endswith(".yaml") or source_file.endswith('.yml'):
-                    self.source = yaml.safe_load(stream)
+                    self.source = yml.load(stream)
                 else:
                     raise CoreError(u"Unable to load source_file. Unknown file format of specified file path: {0}".format(source_file))
 
@@ -83,11 +111,11 @@ class Core(object):
                 if not os.path.exists(f):
                     raise CoreError(u"Provided source_file do not exists on disk : {0}".format(f))
 
-                with open(f, "r") as stream:
+                with open(f, "r", encoding=file_encoding) as stream:
                     if f.endswith(".json"):
                         data = json.load(stream)
                     elif f.endswith(".yaml") or f.endswith(".yml"):
-                        data = yaml.safe_load(stream)
+                        data = yml.load(stream)
                         if not data:
                             raise CoreError(u"No data loaded from file : {0}".format(f))
                     else:
@@ -142,7 +170,7 @@ class Core(object):
             if not os.path.exists(f):
                 raise CoreError(u"Extension file: {0} not found on disk".format(f))
 
-            self.loaded_extensions.append(imp.load_source("", f))
+            self.loaded_extensions.append(SourceFileLoader("", f).load_module())
 
         log.debug(self.loaded_extensions)
         log.debug([dir(m) for m in self.loaded_extensions])
@@ -257,6 +285,12 @@ class Core(object):
 
                 # No exception will should be caught. If one is raised it should bubble up all the way.
                 ret = method(value, rule, path)
+                if ret is not True and ret is not None:
+                    msg = '%s. Path: {path}' % unicode(ret)
+                    self.errors.append(SchemaError.SchemaErrorEntry(
+                                    msg=msg,
+                                    path=path,
+                                    value=None))
 
                 # If False or None or some other object that is interpreted as False
                 if not ret:
@@ -541,7 +575,7 @@ class Core(object):
             if not is_regex_rule:
                 is_present = k in value
             else:
-                is_present = any([re.search(required_regex, v) for v in value])
+                is_present = any([re.search(required_regex, str(v)) for v in value])
 
             # Specifying =: as key is considered the "default" if no other keys match
             if rr.required and not is_present and k != "=":
@@ -632,9 +666,10 @@ class Core(object):
 
         if rule.enum is not None and value not in rule.enum:
             self.errors.append(SchemaError.SchemaErrorEntry(
-                msg=u"Enum '{value}' does not exist. Path: '{path}'",
+                msg=u"Enum '{value}' does not exist. Path: '{path}' Enum: {enum_values}",
                 path=path,
                 value=nativestr(value) if tt['str'](value) else value,
+                enum_values=rule.enum,
             ))
 
         # Set default value
